/**
 * Create a video dialog
 *
 * The `ep.uiVideodialog()` creates a video dialog with a close button.
 *
 * ### Examples
 * Create a simple dialog with some content and a button with a callback function.
 *
 * JavaScript:
 *
 *    // instantiate videodialog widget
 *    $('.ep-helpcenter-video-list-image').uiVideodialog({
 *      type: {
 *        name: 'youtube',
 *        url: $img.data('videourl'),
 *        id: $img.data('videoid'),
 *        ext: $img.data('videoext')
 *      }
 *    });
 *
 * HTML:
 *
 *    <img class="ep-helpcenter-video-list-image" src="https://img.youtube.com/vi/0SclGz7j-p4/hqdefault.jpg" alt="" data-videourl="https://youtu.be/0SclGz7j-p4?list=PL5DlAwWnKPrhv_pGkElTph0M-jTx8whQE" data-videoid="0SclGz7j-p4" data-videoext="?list=PL5DlAwWnKPrhv_pGkElTph0M-jTx8whQE" />
 *
 *
 * @class jQuery.ui.uiVideodialog
 * @extends jQuery.widget
 *
 * @uses jQuery.ui.widget
 * @uses ep
 * @uses ep.fn.video
 * @since 7.30.0
 */

/**
 * Insert a youtube video embedded in an iframe to the dialog.
 * Needs at least a valid youtube video url. Optional: id (= video id), ext (e.g. playlist query parameter)
 *
 * @method _youtube
 * @member jQuery.ui.uiVideodialog
 *
 * @since 7.30.0
 */

/**
 * Insert a vimeo video embedded in an iframe to the dialog.
 * Needs valid html fragment for a iframe with the embedded video.
 *
 * @method _vimeo
 * @member jQuery.ui.uiVideodialog
 *
 * @since 7.34.0
 */

/**
 * Create a dialog for the given rendered video template.
 *
 * @method _createDialog
 * @member jQuery.ui.uiVideodialog
 *
 * @since 7.34.0
 */

/**
 * Open the dialog
 *
 * @method open
 * @member jQuery.ui.uiVideodialog
 *
 * @since 7.30.0
 */

/**
 * Close the dialog.
 *
 * @method close
 * @member jQuery.ui.uiVideodialog
 *
 * @since 7.30.0
 */

/**
 * Removes the dialog functionality completely.
 *
 * @method destroy
 * @member jQuery.ui.uiVideodialog
 *
 * @since 7.30.0
 */

/**
 * See `jQuery.ui.uiVideodialog` for details.
 *
 * @param {Object} options A map of additional options pass to the method.
 *
 * @method uiVideodialog
 * @member jQuery
 *
 * @since 7.30.0
 */

/*
 * @copyright       © Copyright 2006-2018, epages GmbH, All Rights Reserved.
 *
 * @module          ep.ui.videodialog
 */

/*jslint nomen: true*/
/*global define, setTimeout*/
define("ep/ui/videodialog", [
    "jquery",
    "ep",
    "ep/fn/video",

    "ep/ui/dialog",
    "jquery/ui/widget"
], function ($, ep, video) {
    'use strict';

    $.widget('ui.uiVideodialog', {
    //  dialog,

        options : {
        // type: {
        //     name: 'youtube|vimeo',
        //     url: url,
        //     id: '',
        //     ext: '',
        //     html: ''
        // }
        },

        /*
         * Create the widget functionality.
         */
        _create: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            // if options contains a valid video type (e.g. youtube)
            if (o.type && o.type.name && typeof self['_' + o.type.name] === 'function') {
                // register click event
                elem.on('click.uiVideodialog', {widget: self}, function(evt) {
                    // execute corresponding method
                    // e.g. type.name = 'youtube' => self['_youtube'](evt)
                    self['_' + o.type.name](evt);
                });
            }
        },

        /*
         * Opens a dialog that contains an embedded iframe with the youtube video.
         */
        _youtube: function(evt) {
            var _self = evt.data.widget,
                o = _self.options;

            // if dialog was already instantiated => open again
            if (_self.dialog) {
                _self.dialog.uiDialog('open');
            } else {
                // create video dialog with rendered template
                _self._createDialog(video.youtubeIframe({
                    name: o.type.name || 'youtube',
                    embedurl: o.type.embedurl || undefined,
                    url: o.type.url || '',
                    id: o.type.id || '',
                    ext: o.type.ext || ''
                }));

            }
        },

        /*
         * Opens a dialog that contains an embedded iframe with the vimeo video.
         */
        _vimeo: function(evt) {
            var _self = evt.data.widget,
                o = _self.options;

            // if dialog was already instantiated => open again
            if (_self.dialog) {
                _self.dialog.uiDialog('open');
            } else {
                // create video dialog with rendered template
                _self._createDialog(video.vimeoIframe({
                    name: o.type.name || 'vimeo',
                    iframe: o.type.video.html || undefined,
                    id: o.type.video.video_id || ''
                }));
            }

        },

        /*
         * Creates a dialog that contains an embedded iframe with the video.
         */
        _createDialog: function(template) {
            var self = this,
                o = self.options;

            // add dialog instance to data-dialog attribute
            self.dialog = $('<div />')
                // get rendered video iframe html
                .html(template)
                .uiDialog({
                    modal: true,
                    width: '50%',
                    height: 'auto',
                    preventScroll: true
                });
        },


        /*
         * Open the dialog.
         */
        open: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('open');
            }
        },

        /*
         * Close the dialog.
         */
        close: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('close');
            }
        },

        /*
         * Destroy widget.
         */
        destroy: function() {
            var self = this,
                elem = self.element;

            // remove click handler
            elem.off('click.uiVideodialog');
            // remove dialog from document
            if (self.dialog) {
                self.dialog.remove();
            }
            // do inherited stuff
            self._superApply(arguments);
        }

    });

    return ep;
});
